package SLA4;

import java.util.ArrayList;
import java.util.List;

import com.metamaxim.modelscope.callbacks.Instance;

public class IndicatorTimesSubsetInPeriodPercent extends IndicatorPeriodPercent {
	
	@Override
	public Object getIndicatorValue()
	{
		//
		//	Get the total requests during the period
		//
		Instance slaMonitor = this.getInstance("SLA Contract Monitor");
		
		Instance slo = this.getInstance("SLO");
		String period = slo.getString("Norm TimesSubset In Period", "Period Value");

		//
		//	Obtain the low limit and high limit of the targeted response time
		//
		boolean lowInfinity = slo.getBoolean("Norm TimesSubset In Period", "Low Infinity Subset");
		boolean lowInclusive = slo.getBoolean("Norm TimesSubset In Period", "Low Inclusive Subset");
		String  lowTime = slo.getString("Norm TimesSubset In Period", "Low Value Subset");
		boolean highInfinity = slo.getBoolean("Norm TimesSubset In Period", "High Infinity Subset");
		boolean highInclusive = slo.getBoolean("Norm TimesSubset In Period", "High Inclusive Subset");
		String  highTime = slo.getString("Norm TimesSubset In Period", "High Value Subset");

		//
		//	Get the attribute of the target attribute
		//
		String measurementAttributeName = this.getString("Measurement Attribute");
		
		//
		//	Get all the values, regardless whether they are within limits or not
		//
		List<Long> durations = getDurations(slaMonitor, period, measurementAttributeName);
		if (durations.size() == 0)
			return 0;
		
		//
		//	Get only the values that are within limits
		//
		List<Long> within    = getDurationWithinLimits(lowInclusive, lowInfinity? null: lowTime, durations, highInclusive, highInfinity ? null: highTime);

		//
		//	Calculate the percentage that is within limits
		//
		return Math.round(100f*(float)within.size()/(float)durations.size());
	}

	/**
	 *	Get all the duration values
	 * 
	 * @param slaMonitor
	 * @param period
	 * @param measurementAttributeName
	 * @return
	 */
	protected List<Long> getDurations(Instance slaMonitor, String period, String measurementAttributeName)
	{
		List <Long> durations = new ArrayList<Long>();
		
		List<Instance> measurements = this.getMeasurementsInPeriod(slaMonitor, period);
		for (Instance measurement: measurements)
		{
			durations.add( new Period(measurement.getString(measurementAttributeName)).getTime() );
		}
		
		return durations;
	}
	
	/**
	 *	Get all the durations that are within value limits
	 * 
	 * @param lowInclusive
	 * @param lowLimit
	 * @param durations
	 * @param highInclusive
	 * @param highLimit
	 * @return
	 */
	protected List<Long> getDurationWithinLimits(boolean lowInclusive, String lowLimit, List<Long>durations, boolean highInclusive, String highLimit)
	{
		List<Long> within = new ArrayList<Long>();
		for (long value: durations)
		{
			if (isWithin(lowInclusive, lowLimit, value, highInclusive, highLimit))
				within.add(value);
		}
		
		return within;
	}
	
	/**
	 * Returns true if the value is within limits.
	 * 
	 * @param lowInclusive
	 * @param lowLimit
	 * @param value
	 * @param highInclusive
	 * @param highLimit
	 * @return
	 */
	protected boolean isWithin(boolean lowInclusive, String lowLimit, long value, boolean highInclusive, String highLimit)
	{
		return isHigher(lowInclusive, lowLimit, value) && isLower(highInclusive, highLimit, value);
	}
	
	/**
	 * Returns true if the value is higher (or equal) than the low limit.
	 * 
	 * @param lowInclusive
	 * @param lowLimit
	 * @param value
	 * @return
	 */
	protected boolean isHigher(boolean lowInclusive, String lowLimit, long value)
	{
		if (lowLimit == null || lowLimit.length() == 0)
			return true;
		
		long low = new Period(lowLimit).getTime();
		if (lowInclusive)
		{
			return value >= low;
		}

		return value > low;
	}
	
	/**
	 * Returns true if the value is lower (or equal) than the high limit.
	 * 
	 * @param highInclusive
	 * @param highLimit
	 * @param value
	 * @return
	 */
	protected boolean isLower(boolean highInclusive, String highLimit, long value)
	{
		if (highLimit == null || highLimit.length() == 0)
			return true;
		
		long high = new Period(highLimit).getTime();
		if (highInclusive)
		{
			return value <= high;
		}

		return value < high;		
	}

}
