package SLA4;

/**
 * Object to compare a value with a specified range.
 * 
 * The low value above which there is no violation
 * Syntax: [>|=]9+[.9+] | <Empty>
 * Examples:
 *				  10.0 implies raw value >= 10.0
 *				 =10.0 implies raw value >= 10.0
 *				 >10.0 implies raw value > 10.0
 *				<Empty> implies the lowest possible value for a double
 *		
 *	The high value below which there is no violation
 *	Syntax: [<|=]9+[.9+] | <Empty>
 *	Examples:
 *				  10.0 implies raw value <= 10.0
 *				 =10.0 implies raw value <= 10.0
 *				 <10.0 implies raw value < 10.0
 *				<Empty> implies the highest possible value for a double
 *
 * @author Cor Hofman
 *
 */
public abstract class Range {

	private char lowOperand;
	private Object low;
	private char highOperand;
	private Object high;
	
	private DataTypeAbstraction dta;

	/**
	 *  Create the range object.
	 *  
	 * @param sLow
	 * @param sHigh
	 */
	public Range(String sLow, String sHigh)
	{
		this.dta = this.createDta();
		
		//
		//	Find out what the value range is, that this indicator is interested in
		//
		this.setLowOperand('='); //  inclusive lower value
		if (sLow != null && sLow.trim().length() > 0)
		{
			sLow = sLow.trim();
			switch (sLow.toCharArray()[0])
			{
			case '>':
				this.setLowOperand('>');
				sLow = sLow.substring(1);	// Strip
				break;
			}
		}
	
		this.setHighOperand('=');
		if (sHigh != null && sHigh.trim().length() > 0)
		{
			sHigh = sHigh.trim();
			switch (sHigh.toCharArray()[0])
			{
			case '<':
				this.setHighOperand('<');
				sHigh = sHigh.substring(1);	// Strip
				break;
			}
		}
		
		//
		//	Initialize range with default low and high values.
		//
		this.setLow(dta.getLowest());
		this.setHigh(dta.getHighest());

		//
		//	Optionally override the low and high defaults of the range.
		//
		if (sLow != null && sLow.trim().length() > 0)
			setLow(dta.convert(sLow));
		if (sHigh != null && sHigh.trim().length() > 0)
			setHigh(dta.convert(sHigh));		
	}
	
	/**
	 * Get the DataTypeAbstraction object created.
	 * @return
	 */
	protected abstract DataTypeAbstraction createDta();
	
	/**
	 * Returns the relevant DataTypeAbstraction object.
	 * 
	 * @return
	 */
	public DataTypeAbstraction getDta()
	{
		return this.dta;
	}
	
	/**
	 * Returns true if the value is within the range.
	 * 
	 * @param value
	 * @return
	 */
	public boolean isWithin(Object value)
	{
		return greaterThan(value) && smallerThan(value);
	}

	/**
	 *  Gets the comparison operand for the low value.
	 *  
	 * @param highOperand
	 */
	protected char getLowOperand() {
		return lowOperand;
	}


	/**
	 *  Sets the comparison operand for the low value.
	 *  
	 * @param lowOperand
	 */
	protected void setLowOperand(char lowOperand) {
		this.lowOperand = lowOperand;
	}


	/**
	 * Returns the low value of the range.
	 * 
	 * @return
	 */
	protected Object getLow() {
		return low;
	}


	/**
	 * Assigns the low value of the range.
	 * 
	 * @param low
	 */
	protected void setLow(Object low) {
		this.low = low;
	}


	/**
	 *  Gets the comparison operand for the high value.
	 * 
	 * @return
	 */
	protected char getHighOperand() {
		return highOperand;
	}

	/**
	 *  Sets the comparison operand for the high value.
	 *  
	 * @param highOperand
	 */
	protected void setHighOperand(char highOperand) {
		this.highOperand = highOperand;
	}


	/**
	 * Returns the high value set.
	 * @return
	 */
	protected Object getHigh() {
		return high;
	}


	/**
	 * Sets the high value.
	 * 
	 * @param high
	 */
	protected void setHigh(Object high) {
		this.high = high;
	}


	/**
	 * Returns true if the value is above the low value of the range
	 * @param value
	 * @return
	 */
	public boolean greaterThan(Object value)
	{
		boolean result = false;
		switch (this.getLowOperand())
		{
		case '>':
			result = dta.gt(value, this.getLow());
		case '=':
		default:
			
			result = dta.ge(value, this.getLow());
		}
		
		return result;
	}

	/**
	 * Returns true if the value is below the high value of the range.
	 * @param value
	 * @return
	 */
	public boolean smallerThan(Object value)
	{
		boolean result = false;
		
		switch (this.getHighOperand())
		{
		case '<':
			result = dta.lt(value, this.getHigh());
			break;
		case '=':
		default:
			result = dta.le(value, this.getHigh());
		}
		
		return result;
	}

	/**
	 * Returns an initialized Range object of the requested type.
	 * If the type is unknown a RangeDouble is returned.
	 * Which might be wrong :-(
	 * 
	 * @param type
	 * @param low
	 * @param high
	 * @return
	 */
	public static Range newInstance(String type, String low, String high)
	{
		switch (type)
		{
		case "Double":
			return new RangeDouble(low, high);
		case "Integer":
		case "Long":
			return new RangeInteger(low, high);
		case "Period":
			return new RangePeriod(low, high);
		}
		
		return new RangeDouble(low, high);
	}
	
	public static String toString(boolean inclusiveLow, boolean infinityLow, Object valueLow, 
								  boolean inclusiveHigh, boolean infinityHigh, Object valueHigh)
	{
		return "[" + (infinityLow ? "infinite" : toStringInclusiveLow(inclusiveLow) + valueLow) + " - " + 
					 (infinityHigh ? "infinite" : toStringInclusiveHigh(inclusiveHigh) + valueHigh) + "]";
	}
	
	private static String toStringInclusiveLow(boolean inclusive)
	{
		return inclusive ? "=" : "";
	}

	private static String toStringInclusiveHigh(boolean inclusive)
	{
		return inclusive ? "=" : "";
	}

}
